@@ -243,7 +243,7 @@ yqstep (unsigned int Yres, unsigned int bpp)
 /* The functions below return the quantized value of x,y in scanners dots
    aka 1/1200 of an inch */
 
-SANE_Int 
+static SANE_Int 
 xquant (double x, unsigned int Xres, unsigned int bpp, int dir)
 {
   double tmp;
@@ -261,7 +261,7 @@ xquant (double x, unsigned int Xres, unsigned int bpp, int dir)
 
 
 
-SANE_Int 
+static SANE_Int 
 yquant (double y, unsigned int Yres, int dir)
 {
   double tmp;
@@ -1053,7 +1053,8 @@ calc_parameters (Apple_Scanner * s)
 
 
 
-SANE_Status gamma_update(SANE_Handle handle)
+static SANE_Status
+gamma_update(SANE_Handle handle)
 {
 Apple_Scanner *s = handle;
 
@@ -1132,7 +1133,7 @@ return SANE_STATUS_GOOD;
 }
 
 
-SANE_Status
+static SANE_Status
 mode_update (SANE_Handle handle, char *val)
 {
   Apple_Scanner *s = handle;
@@ -1347,6 +1348,7 @@ init_options (Apple_Scanner * s)
       mode_list[5]=NULL;
       break;
     default:
+      break;
     }
 
 
@@ -2431,7 +2433,7 @@ sane_read (SANE_Handle handle, SANE_Byte * buf, SANE_Int max_len,
 #endif  
 
   u_int8_t result[12];
-  ssize_t size;
+  size_t size;
   SANE_Int data_length = 0;
   SANE_Int data_av = 0;
   SANE_Int offset = 0;
@@ -2501,7 +2503,8 @@ sane_read (SANE_Handle handle, SANE_Byte * buf, SANE_Int max_len,
 
       if (data_length)
 	{
-	  if (result[3] & 1)	/* Scanner Blocked: Retrieve data */
+	  /* if (result[3] & 1)	Scanner Blocked: Retrieve data */
+	  if ((result[3] & 1) || data_av)
 	    {
 	      DBG (IO_MESSAGE,
 		   "sane_read: (status) Available in scanner buffer %u.\n",
@@ -2532,7 +2535,7 @@ sane_read (SANE_Handle handle, SANE_Byte * buf, SANE_Int max_len,
 	      status=sanei_scsi_cmd (s->fd,reserve,sizeof(reserve),0,0);
 	      DBG(IO_MESSAGE,"Reserving... status:= %d\n",status);
 	      }
-#endif RESERVE_RELEASE_HACK
+#endif /* RESERVE_RELEASE_HACK */
 
 	      status = sanei_scsi_cmd (s->fd, read, sizeof (read),
 				       buf + offset, &size);
@@ -2544,7 +2547,7 @@ sane_read (SANE_Handle handle, SANE_Byte * buf, SANE_Int max_len,
 	      status=sanei_scsi_cmd (s->fd,release,sizeof(release),0,0);
 	      DBG(IO_MESSAGE,"Releasing... status:= %d\n",status);
 	      }
-#endif RESERVE_RELEASE_HACK
+#endif /* RESERVE_RELEASE_HACK */
 
 
 	      if (Pseudo8bit)
