@@ -51,18 +51,16 @@
    cpinkham@sh001.infi.net
    http://www4.infi.net/~cpinkham/sane/sane-artec-doc.html
    *********************************************************************
-*/
+ */
 
 #ifndef artec_h
 #define artec_h
 
-#define BACKEND_NAME	artec
-
-#define ARTEC_MAJOR 0
-#define ARTEC_MINOR 3
-
 #include <sys/types.h>
 
+#define MIN(a,b) (((a) < (b)) ? (a) : (b))
+#define MAX(a,b) (((a) > (b)) ? (a) : (b))
+
 typedef enum
   {
     OPT_NUM_OPTS = 0,
@@ -74,6 +72,7 @@ typedef enum
     OPT_RESOLUTION_BIND,
     OPT_PREVIEW,
     OPT_GRAY_PREVIEW,
+    OPT_NEGATIVE,
 
     OPT_GEOMETRY_GROUP,
     OPT_TL_X,			/* top-left x */
@@ -82,9 +81,12 @@ typedef enum
     OPT_BR_Y,			/* bottom-right y */
 
     OPT_ENHANCEMENT_GROUP,
+    OPT_QUALITY_CAL,
     OPT_CONTRAST,
     OPT_THRESHOLD,
     OPT_BRIGHTNESS,
+    OPT_HALFTONE_PATTERN,
+    OPT_FILTER_TYPE,
 
     /* must come last */
     NUM_OPTIONS
@@ -106,19 +108,19 @@ typedef enum
   {
     ARTEC_DATA_IMAGE = 0,
     ARTEC_DATA_UNSUPP1,
-    ARTEC_DATA_HALFTONE_PATTERN, /* 2 */
+    ARTEC_DATA_HALFTONE_PATTERN,	/* 2 */
     ARTEC_DATA_UNSUPP3,
-    ARTEC_DATA_RED_SHADING, /* 4 */
-    ARTEC_DATA_GREEN_SHADING, /* 5 */
-    ARTEC_DATA_BLUE_SHADING, /* 6 */
-    ARTEC_DATA_WHITE_SHADING_OPT, /* 7 */
-    ARTEC_DATA_WHITE_SHADING_TRANS, /* 8 */
-    ARTEC_DATA_CAPABILITY_DATA, /* 9 */
-    ARTEC_DATA_DARK_SHADING, /* 10, 0xA */
-    ARTEC_DATA_RED_GAMMA_CURVE, /* 11, 0xB */
-    ARTEC_DATA_GREEN_GAMMA_CURVE, /* 12, 0xC */
-    ARTEC_DATA_BLUE_GAMMA_CURVE, /* 13, 0xD */
-    ARTEC_DATA_ALL_GAMMA_CURVE /* 14, 0xE */
+    ARTEC_DATA_RED_SHADING,	/* 4 */
+    ARTEC_DATA_GREEN_SHADING,	/* 5 */
+    ARTEC_DATA_BLUE_SHADING,	/* 6 */
+    ARTEC_DATA_WHITE_SHADING_OPT,	/* 7 */
+    ARTEC_DATA_WHITE_SHADING_TRANS,	/* 8 */
+    ARTEC_DATA_CAPABILITY_DATA,	/* 9 */
+    ARTEC_DATA_DARK_SHADING,	/* 10, 0xA */
+    ARTEC_DATA_RED_GAMMA_CURVE,	/* 11, 0xB */
+    ARTEC_DATA_GREEN_GAMMA_CURVE,	/* 12, 0xC */
+    ARTEC_DATA_BLUE_GAMMA_CURVE,	/* 13, 0xD */
+    ARTEC_DATA_ALL_GAMMA_CURVE	/* 14, 0xE */
   }
 ARTEC_Read_Data_Type;
 
@@ -129,6 +131,15 @@ typedef enum
   }
 ARTEC_Calibrate_Method;
 
+typedef enum
+  {
+    ARTEC_FILTER_MONO = 0,
+    ARTEC_FILTER_RED,
+    ARTEC_FILTER_GREEN,
+    ARTEC_FILTER_BLUE
+  }
+ARTEC_Filter_Type;
+
 typedef union
   {
     SANE_Word w;
@@ -144,21 +155,22 @@ typedef struct ARTEC_Device
     double width;
     SANE_Range x_range;
     SANE_Range x_dpi_range;
+    SANE_Word *horz_resolution_list;
     double height;
     SANE_Range y_range;
     SANE_Range y_dpi_range;
+    SANE_Word *vert_resolution_list;
     SANE_Range threshold_range;
     SANE_Range contrast_range;
     SANE_Range brightness_range;
-    SANE_Word  setwindow_cmd_size;
-    SANE_Word  calibrate_method;
-    unsigned flags;
+    SANE_Word setwindow_cmd_size;
+    SANE_Word calibrate_method;
 
-    SANE_Bool   support_cap_data_retrieve;
-    SANE_Bool	req_shading_calibrate;
-    SANE_Bool	req_rgb_line_offset;
-    SANE_Bool	req_rgb_char_shift;
-    SANE_Bool	opt_brightness;
+    SANE_Bool support_cap_data_retrieve;
+    SANE_Bool req_shading_calibrate;
+    SANE_Bool req_rgb_line_offset;
+    SANE_Bool req_rgb_char_shift;
+    SANE_Bool opt_brightness;
   }
 ARTEC_Device;
 
@@ -176,11 +188,11 @@ typedef struct ARTEC_Scanner
     size_t line_offset;
 
     /* scan parameters */
-	char	*mode;
-	int		x_resolution;
-	int		y_resolution;
-	int		tl_x;
-	int		tl_y;
+    char *mode;
+    int x_resolution;
+    int y_resolution;
+    int tl_x;
+    int tl_y;
 
     int fd;			/* SCSI filedescriptor */
 
