@@ -3,7 +3,7 @@
 
    dc25.c 
 
-   6/1/98
+   $Id: dc25.c,v 1.1.1.1 1999/08/09 18:05:55 pere Exp $
 
    This file (C) 1998 Peter Fales
 
@@ -105,6 +105,10 @@
 
 #include "dc25.h"
 
+#ifndef PATH_MAX
+# define PATH_MAX	1024
+#endif
+
 #define MAGIC			(void *)0xab730324
 #define DC25_CONFIG_FILE 	"dc25.conf"
 #define THUMBSIZE  ( (CameraInfo.model == 0x25 ) ? 14400 : 5120 )
@@ -112,25 +116,33 @@
 static SANE_Bool is_open = 0;		
 
 static SANE_Byte dc25_opt_image_number = 1;	/* Image to load */
-static SANE_Bool dc25_opt_thumbnails;
-static SANE_Bool dc25_opt_snap;
-static SANE_Bool dc25_opt_lowres;
-static SANE_Fixed dc25_opt_contrast = SANE_FIX(1.75);
-static SANE_Bool dc25_opt_erase;
+static SANE_Bool dc25_opt_thumbnails;		/* Load thumbnails */
+static SANE_Bool dc25_opt_snap;			/* Take new picture */
+static SANE_Bool dc25_opt_lowres;		/* Use low resoluiton */
+#define DC25_OPT_CONTRAST_DEFAULT 1.6
+						/* Contrast enhancement */
+static SANE_Fixed dc25_opt_contrast = SANE_FIX(DC25_OPT_CONTRAST_DEFAULT);
+#define DC25_OPT_GAMMA_DEFAULT 4.5
+						/* Gamma correction (10x) */
+static SANE_Fixed dc25_opt_gamma = SANE_FIX(DC25_OPT_GAMMA_DEFAULT);
+static SANE_Bool dc25_opt_erase;		/* Erase all after download */
+static SANE_Bool dc25_opt_erase_one;		/* Erase one after download */
 static SANE_Bool dumpinquiry;
 
+static SANE_Int  info_flags;
+
 static int tfd;			/* Camera File Descriptor */
 static char *tty_name = "/dev/ttyS0";
-speed_t	tty_baud = B115200;
-char *tmpname;
-char tmpnamebuf[]  = "/tmp/dc25.XXXXXX";
+static speed_t tty_baud = DEFAULT_TTY_BAUD;
+static char *tmpname;
+static char tmpnamebuf[]  = "/tmp/dc25.XXXXXX";
 
 static Dc20Info *dc20_info;
 static Dc20Info CameraInfo;
 
 static SANE_Byte contrast_table[256];
 
-struct pixmap *pp;
+static struct pixmap *pp;
 
 static const SANE_Range percentage_range =
   {
@@ -146,6 +158,13 @@ static const SANE_Range contrast_range =
        16384				/* quantization ~ 0.025 */
   };
 
+static const SANE_Range gamma_range =
+  {
+       0 << SANE_FIXED_SCALE_SHIFT,	/* minimum */
+       10 << SANE_FIXED_SCALE_SHIFT,	/* maximum */
+       16384				/* quantization ~ 0.025 */
+  };
+
 static SANE_Range image_range =
   {
      0,
@@ -230,20 +249,7 @@ static SANE_Option_Descriptor sod[] =
       {NULL}
     },
 
-#define DC25_OPT_CONTRAST 6
-    {
-      "contrast",
-      "Contrast Adjustment",
-      "Values > 1 enhance contrast",
-      SANE_TYPE_FIXED,
-      SANE_UNIT_NONE,
-      sizeof (SANE_Word),
-      SANE_CAP_SOFT_SELECT | SANE_CAP_SOFT_DETECT,
-      SANE_CONSTRAINT_RANGE,
-      {(SANE_String_Const *) &contrast_range}	/* this is ANSI conformant! */
-    },
-
-#define DC25_OPT_ERASE 7
+#define DC25_OPT_ERASE 6
     {
       "erase",
       "Erase",
@@ -256,7 +262,59 @@ static SANE_Option_Descriptor sod[] =
       {NULL}
     },
 
-#define DC25_OPT_DEFAULT 8
+#define DC25_OPT_ERASE_ONE 7
+    {
+      "erase-one",
+      "Erase One",
+      "Erase downloaded picture after downloading",
+      SANE_TYPE_BOOL,
+      SANE_UNIT_NONE,
+      sizeof (SANE_Word),
+      SANE_CAP_SOFT_SELECT | SANE_CAP_SOFT_DETECT,
+      SANE_CONSTRAINT_NONE,
+      {NULL}
+    },
+
+#define DC25_OPT_ENHANCE 8
+    {
+      "",
+      "Image Parameters",
+      "Modifications to image parameters",
+      SANE_TYPE_GROUP,
+      SANE_UNIT_NONE,
+      0,
+      0,
+      SANE_CONSTRAINT_NONE,
+      {NULL}
+    },
+
+#define DC25_OPT_CONTRAST 9
+    {
+      "contrast",
+      "Contrast Adjustment",
+      "Values > 1 enhance contrast",
+      SANE_TYPE_FIXED,
+      SANE_UNIT_NONE,
+      sizeof (SANE_Word),
+      SANE_CAP_SOFT_SELECT | SANE_CAP_SOFT_DETECT,
+      SANE_CONSTRAINT_RANGE,
+      {(SANE_String_Const *) &contrast_range}	/* this is ANSI conformant! */
+    },
+
+#define DC25_OPT_GAMMA 10
+    {
+      "gamma",
+      "Gamma Adjustment",
+      "Larger values make image darker",
+      SANE_TYPE_FIXED,
+      SANE_UNIT_NONE,
+      sizeof (SANE_Word),
+      SANE_CAP_SOFT_SELECT | SANE_CAP_SOFT_DETECT,
+      SANE_CONSTRAINT_RANGE,
+      {(SANE_String_Const *) &gamma_range}	/* this is ANSI conformant! */
+    },
+
+#define DC25_OPT_DEFAULT 11
   {
       "default-enhancements",
       "Defaults",
@@ -285,7 +343,17 @@ static SANE_Parameters parms =
 
 static unsigned char init_pck[] = INIT_PCK;
 
-static struct pkt_speed speeds[] = SPEEDS;
+static struct pkt_speed speeds[] = { {   B9600, { 0x96, 0x00 } },
+				  {  B19200, { 0x19, 0x20 } },
+				  {  B38400, { 0x38, 0x40 } },
+#ifdef B57600
+				  {  B57600, { 0x57, 0x60 } },
+#endif
+#ifdef B115200
+				  { B115200, { 0x11, 0x52 } }
+#endif
+		};
+#define NUM_OF_SPEEDS	(sizeof(speeds) / sizeof(struct pkt_speed))
 
 static struct termios tty_orig;
 
@@ -299,8 +367,11 @@ send_pck (int fd, unsigned char *pck)
 	 * Not quite sure why we need this, but the program works a whole
 	 * lot better (at least on the DC25)  with this short delay.
 	 */
+#ifdef HAVE_USLEEP 
 	usleep (10);
-
+#else
+	sleep (1);
+#endif
 	if (write (fd, (char *)pck, 8) != 8) {
 		DBG (2,"send_pck: error: write returned -1\n");
 		return -1;
@@ -352,7 +423,11 @@ init_dc20 (char *device, speed_t speed)
 	/*
 	 We need the device to be raw. 8 bits even parity on 9600 baud to start.
 	*/
+#ifdef HAVE_cfmakeraw
 	cfmakeraw (&tty_new);
+#else
+	tty_new.c_lflag &= ~(ICANON | ECHO | ISIG);
+#endif
 	tty_new.c_oflag &= ~CSTOPB;
 	tty_new.c_cflag |= PARENB;
 	tty_new.c_cflag &= ~PARODD;
@@ -412,6 +487,7 @@ init_dc20 (char *device, speed_t speed)
 static void 
 close_dc20 (int fd)
 {
+	DBG (127,"close_dc20() called\n");
 	/*
 	 *	Put the camera back to 9600 baud
 	 */
@@ -473,6 +549,7 @@ get_info (int fd)
 	}
 
 	image_range.max = CameraInfo.pic_taken;
+	image_range.min = CameraInfo.pic_taken ? 1 : 0;
 
 	CameraInfo.pic_left = buf[10]<<8|buf[11];
 	
@@ -593,16 +670,14 @@ end_of_data (int fd)
 
 #define SATURATION 1.0
 #define NORM_PERCENTAGE 3
-#define GAMMA 0.35
 
-static int		 columns = HIGH_WIDTH,
+static int			 columns = HIGH_WIDTH,
 				 right_margin = HIGH_RIGHT_MARGIN,
 				 camera_header_size = HIGH_CAMERA_HEADER;
-int				 low_i = -1,
+static int			 low_i = -1,
 				 high_i = -1,
 				 norm_percentage = NORM_PERCENTAGE;
-float			 gamma_value = GAMMA,
-				 saturation = SATURATION,
+static float			 saturation = SATURATION,
 				 rfactor = RFACTOR,
 				 gfactor = GFACTOR,
 				 bfactor = BFACTOR;
@@ -905,18 +980,25 @@ if (verbose) printf ("%s: determine_limits: low_i = %d, high_i = %d\n", __progna
 */
 }
 
+/*
+ * The original dc20ctrl program used a default gamma of 0.35, but I thougt
+ * 0.45 looks better.  In addition, since xscanimage seems to always force 
+ * a resolution of 0.1, I multiply everything by 10 and make the default
+ * 4.5.
+ */
+
 static unsigned char *
 make_gamma_table (int range)
 {
 	int i;
-	double factor = pow (256.0, 1.0 / gamma_value) / range;
+	double factor = pow (256.0, 1.0 / (SANE_UNFIX(dc25_opt_gamma)/10.0)) / range;
 	unsigned char *gamma_table;
 	if ( (gamma_table = malloc (range * sizeof(unsigned char))) == NULL) {
   		DBG (1,"make_gamma_table: can't allocate memory for gamma table\n");
 		return NULL;
 	}
 	for (i = 0; i < range; i++) {
-		int g = pow ( (double)i * factor, gamma_value) + 0.5;
+		int g = pow ( (double)i * factor, (SANE_UNFIX(dc25_opt_gamma)/10.0)) + 0.5;
 /*
 		if (verbose) fprintf (stderr, "%s: make_gamma_table: gamma[%4d] = %3d\n", __progname, i, g);
 */
@@ -1510,6 +1592,12 @@ erase (int fd)
 {
 	int count=0;
 
+	DBG(127,"erase() called for image %d\n",dc25_opt_image_number);
+	erase_pck[3] = dc25_opt_image_number;
+	if ( dc25_opt_erase ) {
+		erase_pck[3]=0;
+	}
+
 	if (send_pck (fd, erase_pck) == -1) {
 		DBG (3,"erase: error: send_pck returned -1\n");
 		return -1;
@@ -1585,6 +1673,8 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 
 	fp = sanei_config_open (DC25_CONFIG_FILE);
 
+	DBG (127, "sane_init() $Id: dc25.c,v 1.1.1.1 1999/08/09 18:05:55 pere Exp $\n");
+
 	if (!fp) {
 		/* default to /dev/ttyS0 instead of insisting on config file */
 		DBG (1, "sane_init:  missing config file '%s'\n",DC25_CONFIG_FILE);
@@ -1619,12 +1709,20 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 				case 38400:
 					tty_baud = B38400;
 					break;
+#ifdef B57600
 				case 57600:
 					tty_baud = B57600;
 					break;
+#endif
+#ifdef B115200
 				case 115200:
 					tty_baud = B115200;
 					break;
+#endif
+				default:
+					DBG (20,"Unknown baud=%d\n",baud);
+					tty_baud = DEFAULT_TTY_BAUD;
+					break;
 				}
 				DBG (20,"Config file baud=%d\n",tty_baud);
 			}
@@ -1632,8 +1730,8 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 				dumpinquiry = SANE_TRUE;
 			}
 		}
+		fclose (fp);
 	}
-	fclose (fp);
 
 	if ( (tfd = init_dc20 (tty_name, tty_baud)) == -1) {
 		return SANE_STATUS_INVAL;
@@ -1763,6 +1861,9 @@ sane_close (SANE_Handle handle)
 		free_pixmap (pp);
 		pp = NULL;
 	}
+
+	close_dc20 ( tfd );
+
 	DBG (127,"sane_close returning\n");
 }
 
@@ -1781,9 +1882,11 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 		     SANE_Action action, void *value,
 		     SANE_Int * info)
 {
-	SANE_Int myinfo = 0;
+	SANE_Int myinfo = info_flags;
 	SANE_Status status;
 
+	info_flags = 0;
+
 	DBG (127, "control_option(handle=%p,opt=%s,act=%s,val=%p,info=%p)\n",
 	handle, sod[option].title, 
 	(action == SANE_ACTION_SET_VALUE ? "SET" : (action == SANE_ACTION_GET_VALUE ? "GET" : "SETAUTO")),
@@ -1820,19 +1923,19 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 				 */
 				parms.format = 
 				 ( CameraInfo.model = 0x25 ) ? SANE_FRAME_RGB : SANE_FRAME_GRAY;
-				parms.bytes_per_line = 10;
+				parms.bytes_per_line = 80*3;
 				parms.pixels_per_line = 80;
 				parms.lines = 60;
 		  	}
 			else {
 				parms.format = SANE_FRAME_RGB;
 				if ( dc20_info->flags.low_res ) {
-					parms.bytes_per_line = 320;
+					parms.bytes_per_line = 320*3;
 					parms.pixels_per_line = 320;
 					parms.lines = 241;
 				}
 				else {
-					parms.bytes_per_line = 500;
+					parms.bytes_per_line = 500*3;
 					parms.pixels_per_line = 500;
 					parms.lines = 373;
 				}
@@ -1859,12 +1962,12 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 				parms.format = SANE_FRAME_RGB;
 
 				if ( dc20_info->flags.low_res ) {
-					parms.bytes_per_line = 320;
+					parms.bytes_per_line = 320*3;
 					parms.pixels_per_line = 320;
 					parms.lines = 241;
 				}
 				else {
-					parms.bytes_per_line = 500;
+					parms.bytes_per_line = 500*3;
 					parms.pixels_per_line = 500;
 					parms.lines = 373;
 				}
@@ -1876,12 +1979,41 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 			dc25_opt_contrast = *(SANE_Word *) value;
 			break;
 
+		case DC25_OPT_GAMMA:
+			dc25_opt_gamma = *(SANE_Word *) value;
+			break;
+
 		case DC25_OPT_ERASE:
 			dc25_opt_erase = !!*(SANE_Word *) value;
+
+			/*
+			 * erase and erase_one are mutually exclusive.  If
+			 * this one is turned on, the other must be off 
+			 */
+			if ( dc25_opt_erase && dc25_opt_erase_one ) {
+				dc25_opt_erase_one = SANE_FALSE;
+				myinfo |= SANE_INFO_RELOAD_OPTIONS;
+			}
+			break;
+
+		case DC25_OPT_ERASE_ONE:
+			dc25_opt_erase_one = !!*(SANE_Word *) value;
+
+			/*
+			 * erase and erase_one are mutually exclusive.  If
+			 * this one is turned on, the other must be off 
+			 */
+			if ( dc25_opt_erase_one && dc25_opt_erase ) {
+				dc25_opt_erase = SANE_FALSE;
+				myinfo |= SANE_INFO_RELOAD_OPTIONS;
+			}
 			break;
 
 		case DC25_OPT_DEFAULT:
-			DBG(1,"Fixme: Set all defaults here!\n");
+
+			dc25_opt_contrast = SANE_FIX(DC25_OPT_CONTRAST_DEFAULT);
+			dc25_opt_gamma = SANE_FIX(DC25_OPT_GAMMA_DEFAULT);
+			myinfo |= SANE_INFO_RELOAD_OPTIONS;
 			break;
 
 		default:
@@ -1915,10 +2047,18 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 			*(SANE_Word *) value = dc25_opt_contrast;
 			break;
 
+		case DC25_OPT_GAMMA:
+			*(SANE_Word *) value = dc25_opt_gamma;
+			break;
+
 		case DC25_OPT_ERASE:
 			*(SANE_Word *)value = dc25_opt_erase;
 			break;
 
+		case DC25_OPT_ERASE_ONE:
+			*(SANE_Word *)value = dc25_opt_erase_one;
+			break;
+
 		default:
 			return SANE_STATUS_INVAL;
 		}
@@ -1927,7 +2067,11 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	case SANE_ACTION_SET_AUTO:
  		switch (option)	{
 		case DC25_OPT_CONTRAST:
-			dc25_opt_contrast = SANE_FIX(1.75);
+			dc25_opt_contrast = SANE_FIX(DC25_OPT_CONTRAST_DEFAULT);
+			break;
+
+		case DC25_OPT_GAMMA:
+			dc25_opt_gamma = SANE_FIX(DC25_OPT_GAMMA_DEFAULT);
 			break;
 
 		default:
@@ -2004,16 +2148,25 @@ sane_start (SANE_Handle handle)
 				return SANE_STATUS_INVAL;
 			} 
 		}
-		usleep (10);
 
+		/*
+		 * Not sure why this delay is needed, but it seems to help:
+		 */
+#ifdef HAVE_USLEEP
+		usleep (10);
+#else
+		sleep(1);
+#endif
 		if ( shoot (tfd) == -1 ) {
 			DBG (1,"Failed to snap new picture\n");
 			return SANE_STATUS_INVAL;
 		} 
 		else {
+			info_flags |= SANE_INFO_RELOAD_OPTIONS;
 			CameraInfo.pic_taken++;
 			CameraInfo.pic_left--;
 			dc25_opt_image_number = CameraInfo.pic_taken;
+			if ( image_range.min == 0 ) image_range.min=1;
 			image_range.max++;
 			sod[DC25_OPT_IMAGE_NUMBER].cap &= ~SANE_CAP_INACTIVE;
 		}
@@ -2045,7 +2198,7 @@ sane_start (SANE_Handle handle)
 		 */
 		parms.format = 
 		 ( CameraInfo.model = 0x25 ) ? SANE_FRAME_RGB : SANE_FRAME_GRAY;
-		parms.bytes_per_line = 10;
+		parms.bytes_per_line = 80*3;  /* 80 pixels, 3 colors */
 		parms.pixels_per_line = 80;
 		parms.lines = 60;
 
@@ -2098,7 +2251,8 @@ sane_start (SANE_Handle handle)
 		if ( buffer[4] == 0 ) { /* hi-res image */
 			DBG (5,"sane_start: hi-res image\n");
 			n = 122;
-			parms.bytes_per_line = 500;
+
+			parms.bytes_per_line = 500*3; /* 3 colors */
 			parms.pixels_per_line = 500;
 			parms.lines = 373;
 
@@ -2108,7 +2262,8 @@ sane_start (SANE_Handle handle)
 		else {
 			n = 61;
 			DBG (5,"sane_start: low-res image\n");
-			parms.bytes_per_line = 320;
+
+			parms.bytes_per_line = 320*3; /* 3 Colors */
 			parms.pixels_per_line = 320;
 			parms.lines = 241;
 
@@ -2158,12 +2313,23 @@ sane_read (SANE_Handle handle, SANE_Byte * data,
 
 	if ( dc25_opt_thumbnails ) {
 		if ( total_bytes_read == THUMBSIZE ) {
-			if ( dc25_opt_erase ) {
+			if ( dc25_opt_erase || dc25_opt_erase_one ) {
+				
 				if ( erase (tfd) == -1 ) {
 					DBG (1,"Failed to erase memory\n");
 					return SANE_STATUS_INVAL;
 				}
-				init_dc20 (tty_name,tty_baud);
+		
+				dc25_opt_erase = SANE_FALSE;
+				dc25_opt_erase_one = SANE_FALSE;
+				info_flags |= SANE_INFO_RELOAD_OPTIONS;
+
+ 				if ( get_info (tfd) == NULL) {
+					DBG (2,"error: could not get info\n");
+					close_dc20 (tfd);
+					return SANE_STATUS_INVAL;
+				}
+DBG(10,"Call get_info!, image range=%d,%d\n",image_range.min,image_range.max);
 			}
 			return SANE_STATUS_EOF;
 		}
@@ -2202,7 +2368,7 @@ sane_read (SANE_Handle handle, SANE_Byte * data,
 	}
 	else {
 		int i;
-		int filesize = parms.bytes_per_line*3*parms.lines;
+		int filesize = parms.bytes_per_line*parms.lines;
 
 		/* 
 		 * If outbytes is zero, then this is the first time
@@ -2237,14 +2403,21 @@ sane_read (SANE_Handle handle, SANE_Byte * data,
 			free_pixmap (pp);
 			pp = NULL;
 
-			if ( dc25_opt_erase ) {
+			if ( dc25_opt_erase || dc25_opt_erase_one ) {
 				if ( erase (tfd) == -1 ) {
 					DBG (1,"Failed to erase memory\n");
 					return SANE_STATUS_INVAL;
 				}
 			}
 
-			init_dc20 (tty_name,tty_baud);
+			if ( get_info (tfd) == NULL) {
+				DBG (2,"error: could not get info\n");
+				close_dc20 (tfd);
+				return SANE_STATUS_INVAL;
+			}
+DBG(10,"Call get_info!, image range=%d,%d\n",image_range.min,image_range.max);
+
+			get_info (tfd);
 			return SANE_STATUS_EOF;
 		}
 
@@ -2274,7 +2447,6 @@ sane_cancel (SANE_Handle handle)
 	DBG (127,"sane_cancel() called\n");
 	started = SANE_FALSE;
 
-	close_dc20 ( tfd );
 }
 
 SANE_Status
