@@ -104,7 +104,7 @@
 
 #define	EPSON_LEVEL_DEFAULT	EPSON_LEVEL_B3
 
-EpsonCmdRec epson_cmd[] =
+static EpsonCmdRec epson_cmd[] =
 {
 /*
          request identity
@@ -748,7 +748,8 @@ attach (const char *dev_name, Epson_Device * *devp)
       if (buf[0] != TYPE_PROCESSOR
 	  || strncmp (buf + 8, "EPSON", 5) != 0
 	  || (strncmp (buf + 16, "SCANNER ", 8) != 0
-	      && strncmp (buf + 14, "SCANNER ", 8) != 0))
+	      && strncmp (buf + 14, "SCANNER ", 8) != 0
+	      && strncmp (buf + 16, "Perfection", 10) != 0))
 	{
 	  DBG (1, "attach: device doesn't look like an Epson scanner\n");
 	  myclose (s);
@@ -1145,7 +1146,8 @@ sane_open (SANE_String_Const devicename, SANE_Handle * handle)
   if (devicename[0] == '\0')
     devicename = dummy_dev.sane.name;
 
-  if (strcmp (devicename, dummy_dev.sane.name) != 0)
+  if (!devicename || devicename[0] == '\0'
+      || strcmp (devicename, dummy_dev.sane.name) != 0)
     return SANE_STATUS_INVAL;
 
   s = calloc (1, sizeof (Epson_Scanner));
