@@ -133,6 +133,7 @@ typedef struct Microtek_Info {
   char vendor_id[9];
   char model_name[17];
   char revision_num[5];
+  char vendor_string[21];
   SANE_Byte device_type;
   SANE_Byte SCSI_firmware_ver_major;
   SANE_Byte SCSI_firmware_ver_minor;
@@ -210,6 +211,7 @@ typedef struct Microtek_Info {
   SANE_Int exposure_vals;
   SANE_Int min_exposure;
   SANE_Int max_exposure;
+  SANE_Byte does_expansion;     /* does expanded-mode expansion internally? */
 } Microtek_Info;
 
 
@@ -267,7 +269,12 @@ typedef struct Microtek_Scanner {
   
   SANE_Option_Descriptor sod[RNUM_OPTIONS]; /* option list for session   */
   Microtek_Option_Value val[RNUM_OPTIONS];  /* option values for session */
-  SANE_Int gamma_table[4][256];
+
+  /*  SANE_Int gamma_table[4][256];*/
+  SANE_Int *gray_lut;
+  SANE_Int *red_lut;
+  SANE_Int *green_lut;
+  SANE_Int *blue_lut;
 
   SANE_Range res_range;      /* range desc. for resolution */
   SANE_Range exp_res_range;  /* range desc. for exp. resolution */
@@ -333,6 +340,7 @@ typedef struct Microtek_Scanner {
   /*  SANE_Int lines;        lines, left to scan     */
   SANE_Int planes;      /* color planes            */
 
+  SANE_Bool doexpansion;
   double exp_aspect;
   SANE_Int dest_pixel_bpl;
   SANE_Int dest_ppl;
