@@ -200,12 +200,14 @@ static const SANE_Range odd_bw_x_range = {1, 335, 2};
 static const SANE_Range bw_y_range = {0, 241, 1}; 
 static const SANE_Range odd_bw_y_range = {1, 242, 1}; 
 
-#ifdef __linux__
+#if defined(__linux__) || defined (HAVE_SYS_HW_H)
 
-#if __GNU_LIBRARY__ - 0 < 6
-# include <asm/io.h>
-#else
-# include <sys/io.h>
+#ifdef HAVE_SYS_IO_H
+# include <sys/io.h>	/* GNU libc based Linux */
+#elif HAVE_ASM_IO_H
+# include <asm/io.h>	/* older Linux */
+#elif HAVE_SYS_HW_H
+# include <sys/hw.h>	/* OS/2 */
 #endif
 
 #define read_lpdata(d)		inb ((d)->port)
@@ -271,18 +273,27 @@ qc_wait (QC_Device * q)
 static SANE_Status
 qc_lock_wait (QC_Device * q, int wait)
 {
+#ifdef F_SETLK
   struct flock sfl;
+#endif
 
   DBG(3, "qc_lock_wait: acquiring lock for 0x%x\n", q->port);
 
+#ifdef F_SETLK
   memset (&sfl, 0, sizeof (sfl));
+#endif
 
   if (q->lock_fd < 0)
     {
       char lockfile[128];
 
       sprintf (lockfile, "/tmp/LOCK.qcam.0x%x", q->port);
-      q->lock_fd = open (lockfile, O_WRONLY | O_CREAT, 0666);
+      q->lock_fd = open (lockfile,
+			 O_WRONLY | O_CREAT
+#ifndef F_SETLK
+			 | O_EXCL
+#endif
+			 , 0666);
       if (q->lock_fd < 0)
 	{
 	  DBG(1, "qc_lock_wait: failed to open %s (%s)\n",
@@ -292,6 +303,7 @@ qc_lock_wait (QC_Device * q, int wait)
 
     }
 
+#ifdef F_SETLK
   sfl.l_type = F_WRLCK;
   if (fcntl (q->lock_fd, wait ? F_SETLKW : F_SETLK, &sfl) != 0)
     {
@@ -299,6 +311,7 @@ qc_lock_wait (QC_Device * q, int wait)
 	  strerror (errno));
       return SANE_STATUS_INVAL;
     }
+#endif
 
   DBG(3, "qc_lock_wait: got lock for 0x%x\n", q->port);
   return SANE_STATUS_GOOD;
@@ -307,16 +320,21 @@ qc_lock_wait (QC_Device * q, int wait)
 static SANE_Status
 qc_unlock (QC_Device *q)
 {
+#ifdef F_SETLK
   struct flock sfl;
+#endif
 
   DBG(3, "qc_lock_wait: releasing lock for 0x%x\n", q->port);
 
+#ifdef F_SETLK
   memset (&sfl, 0, sizeof (sfl));
+#endif
 
   if (q->lock_fd < 0)
     return SANE_STATUS_INVAL;		 /* port was not locked */
 
   /* clear the exclusive lock */
+#ifdef F_SETLK
   sfl.l_type = F_UNLCK;
   if (fcntl (q->lock_fd, F_SETLK, &sfl) != 0)
     {
@@ -324,6 +342,14 @@ qc_unlock (QC_Device *q)
 	  strerror (errno));
       return SANE_STATUS_INVAL;
     }
+#else
+  {
+    char lockfile[128];
+
+    sprintf (lockfile, "/tmp/LOCK.qcam.0x%x", q->port);
+    unlink (lockfile);
+  }
+#endif
   return SANE_STATUS_GOOD;
 }
 
