@@ -51,6 +51,19 @@
 #include "sys/types.h"
 
 
+/* --------------------------------------------------------------------------------------------------------- */
+/* COMPILER OPTIONS: */
+
+
+#define UMAX_HIDE_UNUSED
+/* #define SANE_UMAX_DEBUG_S12 */
+/* #define PREVIEW_FIX_ON */
+
+/* #define UMAX_SHADING_TYPE_SELECTABLE */
+/* #define UMAX_SPEED_SELECTABLE */
+/* #define UMAX_CALIBRATION_MODE_SELECTABLE */
+
+                                         
 /* --------------------------------------------------------------------------------------------------------- */
 
 
@@ -81,9 +94,9 @@ enum Umax_Option
 
     OPT_ENHANCEMENT_GROUP,
 
-    OPT_TEN_BIT_MODE,								        /* use 10 bit output */
-    OPT_TWELVE_BIT_MODE,							        /* use 12 bit output */
+    OPT_BIT_DEPTH,								         /* output bit depth */
     OPT_QUALITY,								      /* quality calibration */
+    OPT_DOR,									/* double optical resolution */
     OPT_WARMUP,
     OPT_RGB_BIND,					 /* use same rgb-values for each color in color-mode */
 
@@ -135,10 +148,19 @@ enum Umax_Option
     OPT_SELECT_EXPOSURE_TIME,
     OPT_SELECT_LAMP_DENSITY,
 
+#ifdef UMAX_SPEED_SELECTABLE
     OPT_SLOW,
     OPT_SMEAR,
+#endif
+
+#ifdef UMAX_CALIBRATION_MODE_SELECTABLE
     OPT_CALIB_MODE,
+#endif
+
+#ifdef UMAX_SHADING_TYPE_SELECTABLE
     OPT_SHADING_TYPE,
+#endif
+
     OPT_PREVIEW,					/* preview, sets preview-bit and bind x/y-resolution */
 
     /* must come last: */
@@ -204,21 +226,26 @@ typedef struct Umax_Device
 
   int			three_pass;					  /* used in RGB-mode if 3-pass => 1 */
   int			three_pass_color;			  /* select color for scanning in 3pass mode */
-  int			row_len;					    /* len of one scan-line in bytes */
+  unsigned int		row_len;					    /* len of one scan-line in bytes */
   unsigned int		max_value;							/* used for pnm-file */
 
   /* data defined by inquiry */
   int			inquiry_len;					   /* length of inquiry return block */
   int			inquiry_wdb_len;				/* length of window descriptor block */
-  int			inquiry_optical_res;			     /* optical resolution form read_inquiry */
+  int			inquiry_optical_res;			     		       /* optical resolution */
   int			inquiry_x_res;						     /* maximum x-resolution */
   int			inquiry_y_res;						     /* maximum y-resolution */
+  int			inquiry_dor_optical_res;			  /* optical resolution for dor mode */
+  int			inquiry_dor_x_res;				/* maximum x-resolution for dor mode */
+  int			inquiry_dor_y_res;				/* maximum y-resolution for dor mode */
   double		inquiry_fb_width;					  /* flatbed width in inches */
   double		inquiry_fb_length;					 /* flatbed length in inches */
   double		inquiry_uta_width;				     /* transparency width in inches */
   double		inquiry_uta_length;				    /* transparency length in inches */
   double		inquiry_dor_width;				/* double resolution width in inches */
   double		inquiry_dor_length;			       /* double resolution length in inches */
+  double		inquiry_dor_x_off;			     /* double resolution x offset in inches */
+  double		inquiry_dor_y_off;			     /* double resolution y offset in inches */
 
   int			inquiry_exposure_adj;				/* 1 if exposure adjust is supported */
   int			inquiry_exposure_time_step_unit;		  /* exposure time unit in micro sec */ 
@@ -229,9 +256,9 @@ typedef struct Umax_Device
   int			inquiry_exposure_time_h_min;		       /* exposure tine minimum for halftone */
   int			inquiry_exposure_time_h_fb_def;	       /* exposure time default for halftone/flatbed */
   int			inquiry_exposure_time_h_uta_def;	   /* exposure time default for halftone/uta */
-  int			inquiry_exposure_time_g_min;		      /* exposure tine minimum for greyscale */
-  int			inquiry_exposure_time_g_fb_def;	      /* exposure time default for greyscale/flatbed */
-  int			inquiry_exposure_time_g_uta_def;	  /* exposure time default for greyscale/uta */
+  int			inquiry_exposure_time_g_min;		      /* exposure tine minimum for grayscale */
+  int			inquiry_exposure_time_g_fb_def;	      /* exposure time default for grayscale/flatbed */
+  int			inquiry_exposure_time_g_uta_def;	  /* exposure time default for grayscale/uta */
   int			inquiry_exposure_time_c_min;			  /* exposure tine minimum for color */
   int			inquiry_exposure_time_c_fb_def_r;     /* exposure time default for color red/flatbed */
   int			inquiry_exposure_time_c_fb_def_g;   /* exposure time default for color green/flatbed */
@@ -263,7 +290,7 @@ typedef struct Umax_Device
   int			inquiry_one_pass_color;					     /* 1 = 1 pass supported */
   int			inquiry_three_pass_color;				     /* 1 = 3 pass supported */
   int			inquiry_color;						 /* 1 = color mode supported */
-  int			inquiry_grey;					     /* 1 = greyscale mode supported */
+  int			inquiry_gray;					     /* 1 = grayscale mode supported */
   int			inquiry_halftone;				      /* 1 = halftone mode supported */
   int			inquiry_lineart;				       /* 1 = lineart mode supported */
 
@@ -278,6 +305,10 @@ typedef struct Umax_Device
   int			inquiry_fb_uta_color_arrangement;		    /* line arrangement for fb & uta */
   int			inquiry_adf_color_arrangement;				 /* line arrangement for adf */
 
+  int			relevant_optical_res;			     		       /* optical resolution */
+  int			relevant_max_x_res;					     /* maximum x-resolution */
+  int			relevant_max_y_res;					     /* maximum y-resolution */
+
   /* selected data */
 
   int			use_exposure_time_min;					   /*  exposure tine minimum */
@@ -286,25 +317,27 @@ typedef struct Umax_Device
   int			use_exposure_time_def_b;				    /* exposure time default */
 
   int			wdb_len;						   /* use this length of WDB */
-  double		width;						      /* use this width of scan-area */
-  double		length;						     /* use this length of scan-area */
-  int			width_in_pixels;				 /* thats the wanted width in pixels */
-  int			height_in_pixels;				/* thats the wanted length in pixels */
-  int			phys_width;				 /* thats the width in pixels at 1200pt/inch */
-  int			phys_height;				/* thats the length in pixels at 1200pt/inch */
+  double		maxwidth;					      /* use this width of scan-area */
+  double		maxlength;					     /* use this length of scan-area */
+  unsigned int		width_in_pixels;				 /* thats the wanted width in pixels */
+  unsigned int		length_in_pixels;				/* thats the wanted length in pixels */
+  unsigned int		scanwidth;		       /* thats the width in pixels at x_coordinate_base dpi */
+  unsigned int		scanlength;		      /* thats the length in pixels at y_coordinate_base dpi */
+  unsigned int		bytes_per_color;					     /* bytes per each color */
 
-  int			x_resolution;					     /* scan-resolution for x in dpi */
-  int			y_resolution;					     /* scan-resolution for y in dpi */
+  unsigned int		x_resolution;					     /* scan-resolution for x in dpi */
+  unsigned int		y_resolution;					     /* scan-resolution for y in dpi */
   double		scale_x;					  /* x-scaling of optical resolution */
   double		scale_y;					  /* y-scaling of optical resolution */
   int			upper_left_x;			     /* thats the left edge in points at 1200pt/inch */
   int			upper_left_y;			      /* thats the top edge in points at 1200pt/inch */
 
-  int			x_coordinate_base;			      /* x base in pixels/inch, normaly 1200 */
-  int			y_coordinate_base;			      /* y base in pixels/inch, normaly 1200 */
+  unsigned int		x_coordinate_base;			      /* x base in pixels/inch, normaly 1200 */
+  unsigned int		y_coordinate_base;			      /* y base in pixels/inch, normaly 1200 */
 
-  int			bits_per_pixel;					/* 1 = 24bpp, 4 = 30 bpp, 5 = 36 bpp */
-  int			gamma_input_bits;				/* 1 = 24bpp, 4 = 30 bpp, 5 = 36 bpp */
+  unsigned int		bits_per_pixel;						 /* number of bits per pixel */
+  int			bits_per_pixel_code;				/* 1 = 24bpp, 4 = 30 bpp, 8 = 36 bpp */
+  int			gamma_input_bits_code;				/* 1 = 24bpp, 4 = 30 bpp, 8 = 36 bpp */
   int			set_auto;					    /* 0 or 1, don't know what it is */
   int			preview;							     /* 1 if preview */
   int			quality;					/* 1 = quality_calibration, 0 = fast */
@@ -322,10 +355,10 @@ typedef struct Umax_Device
   int			uta;							       /* 1 if UTA turned on */
   int			calibration;			      /* calibration :0=ignore, 1=driver, 2=by image */
   int			low_byte_first;			 /* 10 bit mode: 0=high byte frist, 1=low byte frist */
-  int			colormode;				     /* LINEART, HALFTONE, GREYSCALE or RGB  */
+  int			colormode;				     /* LINEART, HALFTONE, GRAYSCALE or RGB  */
 #  define LINEART             1
 #  define HALFTONE            2
-#  define GREYSCALE           3
+#  define GRAYSCALE           3
 #  define RGB_LINEART         4
 #  define RGB_HALFTONE        5
 #  define RGB                 6
@@ -351,11 +384,11 @@ typedef struct Umax_Device
   int			shadow_b;					  /* (0) cbhs_range 0-254, each mode */
   int			halftone;						  /* halftone pattern select */
 
-  int			digital_gamma_r;				    /* gamma-select for red and grey */
+  int			digital_gamma_r;				    /* gamma-select for red and gray */
   int			digital_gamma_g;				     /* gamma-select value for green */
   int			digital_gamma_b;				      /* gamma-select value for blue */
 
-  int			analog_gamma_r;						/* analog gamma red and grey */
+  int			analog_gamma_r;						/* analog gamma red and gray */
   int			analog_gamma_g;						       /* analog gamma green */
   int			analog_gamma_b;							/* analog gamma blue */
 
@@ -366,6 +399,13 @@ typedef struct Umax_Device
 
   int			button_pressed;				   /* scan-button on scanner is pressed => 1 */
 
+  int			calibration_area;		      /* define calibration area if no area is given */
+  int			pause_for_color_calibration;	/* pause between start_scan and do_calibration in ms */
+  int			pause_for_gray_calibration;	/* pause between start_scan and do_calibration in ms */
+  int			pause_after_calibration;	 /* pause between do_calibration and read data in ms */
+  int			pause_after_reposition;				    /* pause for repositioning in ms */
+  int			pause_for_moving;	       /* pause for moving scanhead over full scanarea in ms */
+
   int                   RGB_PREVIEW_FIX;
 } Umax_Device;
 
@@ -383,10 +423,11 @@ typedef struct Umax_Scanner
 
   SANE_Option_Descriptor	opt[NUM_OPTIONS];
   Option_Value			val[NUM_OPTIONS];
-  SANE_Int			gamma_table[4][1024];
+  SANE_Int			*gamma_table[4];
   SANE_Int			halftone_pattern[64];
   SANE_Range			gamma_range;
   int				gamma_length;
+  SANE_Range			output_range;
   int				output_bytes;
   SANE_Range			exposure_time_range;
 
