@@ -51,6 +51,7 @@
 #include <sys/param.h>
 #include <sys/socket.h>
 #include <sys/time.h>
+#include <sys/types.h>
 
 #include <sane/sane.h>
 #include <sane/sanei.h>
@@ -66,6 +67,10 @@
 # define IN_LOOPBACK(addr) (addr == 0x7f000001L)
 #endif
 
+#ifndef MAXHOSTNAMELEN
+# define MAXHOSTNAMELEN 120
+#endif
+
 #define SANED_CONFIG_FILE "saned.conf"
 
 /* The cpp that comes with GNU C 2.5 seems to have troubles understanding
@@ -82,7 +87,7 @@
   do						\
     {						\
       if ((debug) >= (level))			\
-	syslog (LOG_DEBUG, (msg), ##args);	\
+	syslog (LOG_DEBUG, msg, ##args);	\
     }						\
   while (0)
 # endif
@@ -150,7 +155,7 @@ auth_callback (SANE_String_Const res,
   memset (username, 0, sizeof (username));
   memset (password, 0, sizeof (password));
 
-  if (can_authorize)
+  if (!can_authorize)
     {
       syslog (LOG_ERR,
 	      "auth_callback(resource=%s) during non-authorizable RPC\n",
@@ -226,11 +231,19 @@ auth_callback (SANE_String_Const res,
 static void
 quit (int signum)
 {
+  static int running = 0;
   int i;
 
   if (signum)
     DBG(1, "received signal %d\n", signum);
 
+  if (running)
+    {
+      DBG(1, "quit is already active, returning\n");
+      return;
+    }
+  running = 1;
+
   for (i = 0; i < num_handles; ++i)
     if (handle[i].inuse)
       sane_close (handle[i].handle);
@@ -273,6 +286,10 @@ get_free_handle (void)
     handle = realloc (handle, num_handles * sizeof (handle[0]));
   else
     handle = malloc (num_handles * sizeof (handle[0]));
+  if (!handle)
+      return -1;
+  memset (handle + last_handle_checked + 1, 0,
+	  ALLOC_INCREMENT*sizeof (handle[0]));
   return get_free_handle ();
 # undef ALLOC_INCREMENT
 }
@@ -301,6 +318,7 @@ decode_handle (Wire *w, const char * op)
   return h;
 }
 
+/* Check hostnames ignoring case as DNS should. */
 static SANE_Status
 check_host (int fd)
 {
@@ -339,12 +357,12 @@ check_host (int fd)
   if (IN_LOOPBACK (ntohl (sin.sin_addr.s_addr)))
     return SANE_STATUS_GOOD;
 
-  if (strcmp (hostname, he->h_name) == 0)
+  if (strcasecmp (hostname, he->h_name) == 0)
     return SANE_STATUS_GOOD;
 
   /* check alias list: */
   for (i = 0; he->h_aliases[i]; ++i)
-    if (strcmp (hostname, he->h_aliases[i]) == 0)
+    if (strcasecmp (hostname, he->h_aliases[i]) == 0)
       return SANE_STATUS_GOOD;
 
   /* must be a remote host: check contents of PATH_NET_CONFIG or
@@ -370,10 +388,10 @@ check_host (int fd)
 	  if (!len)
 	    continue;		/* ignore empty lines */
 
-	  if (strcmp (rhost, he->h_name) == 0)
+	  if (strcasecmp (rhost, he->h_name) == 0 || strcmp (rhost, "+") == 0)
 	    access_ok = 1;
 	  for (i = 0; he->h_aliases[i]; ++i)
-	    if (strcmp (rhost, he->h_aliases[i]) == 0)
+	    if (strcasecmp (rhost, he->h_aliases[i]) == 0)
 	      {
 		access_ok = 1;
 		break;
@@ -399,6 +417,7 @@ init (Wire *w)
   sanei_w_set_dir (w, WIRE_DECODE);
   sanei_w_word (w, &word);		/* decode procedure number */
   sanei_w_init_req (w, &req);
+  w->version = SANEI_NET_PROTOCOL_VERSION;
 
   if (w->status || word != SANE_NET_INIT)
     {
@@ -708,8 +727,13 @@ process_request (Wire *w)
 	if (reply.status == SANE_STATUS_GOOD)
 	  {
 	    h = get_free_handle ();
-	    handle[h].handle = be_handle;
-	    reply.handle = h;
+	    if (h < 0)
+	      reply.status = SANE_STATUS_NO_MEM;
+	    else
+	      {
+		handle[h].handle = be_handle;
+		reply.handle = h;
+	      }
 	  }
 
 	can_authorize = 0;
