@@ -71,6 +71,7 @@
 #define SANE_NAME_RESOLUTION_BIND	"resolution-bind"
 #define SANE_NAME_NEGATIVE		"negative"
 #define SANE_NAME_QUALITY_CAL		"quality-cal"
+#define SANE_NAME_DOR			"double-res"
 #define SANE_NAME_RGB_BIND		"rgb-bind"
 #define SANE_NAME_THRESHOLD		"threshold"
 #define SANE_NAME_ANALOG_GAMMA		"analog-gamma"
@@ -139,6 +140,7 @@
 #define SANE_TITLE_RESOLUTION_BIND	"Bind X and Y resolution"
 #define SANE_TITLE_NEGATIVE		"Negative"
 #define SANE_TITLE_QUALITY_CAL		"Quality calibration"
+#define SANE_TITLE_DOR			"Double Optical Resolution"
 #define SANE_TITLE_RGB_BIND		"Bind RGB"
 #define SANE_TITLE_THRESHOLD		"Threshold"
 #define SANE_TITLE_ANALOG_GAMMA		"Analog gamma correction"
@@ -288,6 +290,7 @@
 #define SANE_DESC_RESOLUTION_BIND	"Use same values for X and Y resolution"
 #define SANE_DESC_NEGATIVE		"Swap black and white"
 #define SANE_DESC_QUALITY_CAL		"Do a quality white-calibration"
+#define SANE_DESC_DOR			"Use lens that doubles optical resolution"
 #define SANE_DESC_RGB_BIND		"In RGB-mode use same values for each color"
 #define SANE_DESC_THRESHOLD		"Select minimum-brightness to get a white point"
 #define SANE_DESC_ANALOG_GAMMA		"Analog gamma-correction"
